package com.yihu.hos.core.http;

import okhttp3.*;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Map;

/**
 * @created Airhead 2016/8/24.
 */
class DefaultClientImpl implements HTTPClient {
    private static final Log log = LogFactory.getLog(DefaultClientImpl.class);

    private OkHttpClient httpClient;

    DefaultClientImpl() {
        this.httpClient = new OkHttpClient();
    }

    public HTTPResponse get(String url) {
        return get(url, null);
    }

    public HTTPResponse get(String url, Map<String, String> params) {
        return get(url, params, null);
    }

    public HTTPResponse get(String url, Map<String, String> params, Map<String, String> headers) {
        try {
            Request.Builder builder = new Request.Builder();
            if (headers != null) {
                builder.headers(Headers.of(headers));
            }
            Request request = builder
                    .url(formatURL(url, params))
                    .build();

            Response response = httpClient.newCall(request).execute();
            if (response.isSuccessful()) {
                return new HTTPResponse(response.code(), response.body().string());
            }
        } catch (IOException ex) {
            log.error(ex.getMessage());
        }

        return new HTTPResponse(500, "");
    }

    public HTTPResponse post(String url) {
        return post(url, null);
    }

    public HTTPResponse post(String url, Map<String, String> params) {
        return post(url, params, null);
    }

    public HTTPResponse post(String url, Map<String, String> params, Map<String, String> headers) {
        try {
            FormBody.Builder fromBodyBuilder = new FormBody.Builder();
            params.forEach(fromBodyBuilder::add);
            RequestBody requestBody = fromBodyBuilder
                    .build();

            Request.Builder builder = new Request.Builder();
            if (headers != null) {
                builder.headers(Headers.of(headers));
            }
            Request request = builder
                    .url(url)
                    .post(requestBody)
                    .build();

            Response response = httpClient.newCall(request).execute();
            if (response.isSuccessful()) {
                return new HTTPResponse(response.code(), response.body().string());
            }
        } catch (IOException ex) {
            log.error(ex.getMessage());
        }

        return new HTTPResponse(500, "");
    }

    public HTTPResponse postFile(String url, String path) {
        return postFile(url, path, null);
    }

    public HTTPResponse postFile(String url, String path, Map<String, String> params) {
        return postFile(url, path, params, null);
    }

    public HTTPResponse postFile(String url, String path, Map<String, String> params, Map<String, String> headers) {
        try {
            File file = new File(path);

            final MediaType textPlain = MediaType.parse("text/plain; charset=utf-8");
            Request.Builder builder = new Request.Builder();
            if (headers != null) {
                builder.headers(Headers.of(headers));
            }
            Request request = builder
                    .url(formatURL(url, params))
                    .post(RequestBody.create(textPlain, file))
                    .build();

            Response response = httpClient.newCall(request).execute();
            if (response.isSuccessful()) {
                return new HTTPResponse(response.code(), response.body().string());
            }
        } catch (IOException ex) {
            log.error(ex.getMessage());
        }

        return new HTTPResponse(500, "");
    }

    public HTTPResponse put(String url) {
        return put(url, null);
    }

    public HTTPResponse put(String url, Map<String, String> params) {
        return put(url, params, null);
    }

    public HTTPResponse put(String url, Map<String, String> params, Map<String, String> headers) {
        try {
            FormBody.Builder fromBodyBuilder = new FormBody.Builder();
            params.forEach(fromBodyBuilder::add);
            RequestBody requestBody = fromBodyBuilder
                    .build();

            Request.Builder builder = new Request.Builder();
            if (headers != null) {
                builder.headers(Headers.of(headers));
            }
            Request request = builder
                    .url(url)
                    .put(requestBody)
                    .build();

            Response response = httpClient.newCall(request).execute();
            if (response.isSuccessful()) {
                return new HTTPResponse(response.code(), response.body().string());
            }
        } catch (IOException ex) {
            log.error(ex.getMessage());
        }

        return new HTTPResponse(500, "");
    }

    public HTTPResponse delete(String url) {
        return delete(url, null);
    }

    public HTTPResponse delete(String url, Map<String, String> params) {
        return delete(url, params, null);
    }

    public HTTPResponse delete(String url, Map<String, String> params, Map<String, String> headers) {
        try {
            FormBody.Builder fromBodyBuilder = new FormBody.Builder();
            params.forEach(fromBodyBuilder::add);
            RequestBody requestBody = fromBodyBuilder
                    .build();

            Request.Builder builder = new Request.Builder();
            if (headers != null) {
                builder.headers(Headers.of(headers));
            }
            Request request = builder
                    .url(url)
                    .delete(requestBody)
                    .build();

            Response response = httpClient.newCall(request).execute();
            if (response.isSuccessful()) {
                return new HTTPResponse(response.code(), response.body().string());
            }
        } catch (IOException ex) {
            log.error(ex.getMessage());
        }

        return new HTTPResponse(500, "");
    }

    public HTTPResponse request(String method, String url, Map<String, String> params, Map<String, String> headers) {
        if (method.equals(POST)) {
            return post(url, params, headers);
        }

        if (method.equals(GET)) {
            return get(url, params, headers);
        }

        if (method.equals(PUT)) {
            return put(url, params, headers);
        }

        if (method.equals(DELETE)) {
            return delete(url, params, headers);
        }

        return get(url, params, headers);
    }

    protected void setHttpClient(OkHttpClient okHttpClient) {
        this.httpClient = okHttpClient;
    }

    private String formatURL(String url, Map<String, String> params) {
        if (params == null) {
            return url;
        }

        final String[] query = {""};
        params.forEach((name, value) -> {
            try {
                query[0] += "&" + name + URLEncoder.encode(value, "UTF-8");
            } catch (UnsupportedEncodingException e) {
                throw new RuntimeException("encode url ");
            }
        });

        if (url.contains("?")) {
            return url + query[0];
        }

        return url + "?" + query[0].substring(1);
    }
}
